import QtQml 2.15
import QtQuick 2.15
import QtQuick.Window 2.15

// after clicking, this will automatically show and hide the window, and well as toggle a loader and dynamically edit a shader
// this should result in a flickering Qt logo to the left, and a red box to the right
// if you run it enough, at some point you will incorrectly see a red box on the left, and/or the Qt logo on the right
Window {
    id: window
    width: 1280
    height: 720
    color: "black"
    visible: true

    Image {
        id: logo
        visible: false
        cache: false
        source: "qrc:/qtlogo.png"
    }

    Image {
        id: logo2
        visible: false
        cache: false
        source: "qrc:/qtlogo.png"
    }

    Loader {
        id: loader
        active: false
        sourceComponent: ShaderEffect {
            width: 200; height: 200
            property variant source: logo
            fragmentShader: "
                varying highp vec2 qt_TexCoord0;
                uniform sampler2D source;
                uniform lowp float qt_Opacity;
                void main() {
                    lowp vec4 tex = texture2D(source, qt_TexCoord0);
                    gl_FragColor = vec4(tex.rgb, tex.a) * qt_Opacity;
                }"
        }
    }


    MouseArea {
        anchors.fill: parent
        onClicked: {
            visibleTimer1.start()
            loaderTimer.start()
        }
    }

    Text {
        anchors.centerIn: parent
        font.pixelSize: 36
        color: "white"
        text: "Click to Start"
    }

    Timer {
        id: visibleTimer1
        interval: 1000
        onTriggered: {
            window.visible = false
            visibleTimer2.start()
        }
    }

    Timer {
        id: visibleTimer2
        interval: 500
        onTriggered: {
            window.visible = true
            visibleTimer1.start()
        }
    }


    Timer {
        id: loaderTimer
        interval: 160
        repeat: true
        onTriggered: {
            if(window.visible === true) {
                loader.active = !loader.active
                effect2.fragmentShader = effect2.fragmentShader.toString().replace("gl_FragColor", "tex.r = tex.r + 0.01;\ngl_FragColor")
            }
        }
    }

    //work around
    /*onVisibleChanged: {
        if(window.visible === true) {
            effect2.fragmentShader = effect2.fragmentShaderString
        }
        else {
            effect2.fragmentShader = ""
        }
    }*/

    // to start, this will be the Qt logo tinted red
    // the loaderTimer above will quickly turn it into a red box
    ShaderEffect {
        id: effect2
        anchors.right: parent.right
        width: 200; height: 200
        property variant source: logo2
        fragmentShader: fragmentShaderString
        property variant fragmentShaderString: "
            varying highp vec2 qt_TexCoord0;
            uniform sampler2D source;
            uniform lowp float qt_Opacity;
            void main() {
                lowp vec4 tex = texture2D(source, qt_TexCoord0);
                gl_FragColor = vec4(tex.r, 0.0, 0.0, tex.a) * qt_Opacity;
            }"
    }
}
